@@ -169,7 +169,7 @@ void SelectEmojiWidget::setupEmojiTab(int index)
                   << "9f-90a3" << "94-46" << "94-45" << "9f-90a2" << "94-37" << "9f-909d"
                   << "9f-909c" << "9f-909e" << "9f-908c" << "84-10" << "91-1" << "94-34"
                   << "80-25" << "94-32" << "81-20" << "9f-908b" << "9f-9084" << "9f-908f"
-                  << "9f-9080" << "9f-9083" << "9f-9085" << "9f-9087" << "9f-9089"
+                  << "9f-9080" << "9f-9083" << "9f-9085" << "9f-9087" << "9f-9089" << "9f-908e"
                   << "9f-9090" << "9f-9093" << "9f-9095" << "9f-9096" << "9f-9081"
                   << "9f-9082" << "9f-90b2" << "9f-90a1" << "9f-908a" << "94-48"
                   << "9f-90aa" << "9f-9086" << "9f-9088" << "9f-90a9" << "9f-90be"
